import { IQueryHandler, QueryHandler } from '@nestjs/cqrs';
import { MatchQuery } from './match.query';
import { Algorithm } from './algorithm.abstract';
import { PassengerOrientedAlgorithm } from './passenger-oriented-algorithm';
import { AlgorithmType } from '../../types/algorithm.types';
import { Inject } from '@nestjs/common';
import { AdRepositoryPort } from '@modules/ad/core/application/ports/ad.repository.port';
import { AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';

@QueryHandler(MatchQuery)
export class MatchQueryHandler implements IQueryHandler {
  constructor(
    @Inject(AD_REPOSITORY) private readonly repository: AdRepositoryPort,
  ) {}
  execute = async (query: MatchQuery): Promise<MatchEntity[]> => {
    let algorithm: Algorithm;
    switch (query.algorithmType) {
      case AlgorithmType.PASSENGER_ORIENTED:
      default:
        algorithm = new PassengerOrientedAlgorithm(query, this.repository);
    }
    return algorithm.match();
  };
}
