import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { DefaultParamsProviderPort } from '../core/application/ports/default-params-provider.port';
import { DefaultParams } from '../core/application/ports/default-params.type';

const DEFAULT_SEATS_PROPOSED = 3;
const DEFAULT_SEATS_REQUESTED = 1;
const DEFAULT_DEPARTURE_TIME_MARGIN = 900;
const DEFAULT_TIMEZONE = 'Europe/Paris';

@Injectable()
export class DefaultParamsProvider implements DefaultParamsProviderPort {
  constructor(private readonly _configService: ConfigService) {}
  getParams = (): DefaultParams => ({
    DRIVER: this._configService.get('ROLE') == 'driver',
    SEATS_PROPOSED:
      this._configService.get('SEATS_PROPOSED') !== undefined
        ? parseInt(this._configService.get('SEATS_PROPOSED') as string)
        : DEFAULT_SEATS_PROPOSED,
    PASSENGER: this._configService.get('ROLE') == 'passenger',
    SEATS_REQUESTED:
      this._configService.get('SEATS_REQUESTED') !== undefined
        ? parseInt(this._configService.get('SEATS_REQUESTED') as string)
        : DEFAULT_SEATS_REQUESTED,
    DEPARTURE_TIME_MARGIN:
      this._configService.get('DEPARTURE_TIME_MARGIN') !== undefined
        ? parseInt(this._configService.get('DEPARTURE_TIME_MARGIN') as string)
        : DEFAULT_DEPARTURE_TIME_MARGIN,
    STRICT: this._configService.get('STRICT_FREQUENCY') == 'true',
    DEFAULT_TIMEZONE:
      this._configService.get('DEFAULT_TIMEZONE') ?? DEFAULT_TIMEZONE,
  });
}
