import { AD_REPOSITORY } from '@modules/ad/ad.di-tokens';
import { MatchQuery } from '@modules/ad/core/application/queries/match/match.query';
import { MatchQueryHandler } from '@modules/ad/core/application/queries/match/match.query-handler';
import { AlgorithmType } from '@modules/ad/core/application/types/algorithm.types';
import { Waypoint } from '@modules/ad/core/application/types/waypoint.type';
import { Frequency } from '@modules/ad/core/domain/ad.types';
import { MatchEntity } from '@modules/ad/core/domain/match.entity';
import { Test, TestingModule } from '@nestjs/testing';

const originWaypoint: Waypoint = {
  position: 0,
  lat: 48.689445,
  lon: 6.17651,
  houseNumber: '5',
  street: 'Avenue Foch',
  locality: 'Nancy',
  postalCode: '54000',
  country: 'France',
};
const destinationWaypoint: Waypoint = {
  position: 1,
  lat: 48.8566,
  lon: 2.3522,
  locality: 'Paris',
  postalCode: '75000',
  country: 'France',
};

const mockAdRepository = {
  getCandidates: jest.fn(),
};

describe('Match Query Handler', () => {
  let matchQueryHandler: MatchQueryHandler;

  beforeAll(async () => {
    const module: TestingModule = await Test.createTestingModule({
      providers: [
        MatchQueryHandler,
        {
          provide: AD_REPOSITORY,
          useValue: mockAdRepository,
        },
      ],
    }).compile();

    matchQueryHandler = module.get<MatchQueryHandler>(MatchQueryHandler);
  });

  it('should be defined', () => {
    expect(matchQueryHandler).toBeDefined();
  });

  it('should return a Match entity', async () => {
    const matchQuery = new MatchQuery({
      algorithmType: AlgorithmType.PASSENGER_ORIENTED,
      driver: false,
      passenger: true,
      frequency: Frequency.PUNCTUAL,
      fromDate: '2023-08-28',
      toDate: '2023-08-28',
      schedule: [
        {
          time: '07:05',
          day: 1,
          margin: 900,
        },
      ],
      strict: false,
      waypoints: [originWaypoint, destinationWaypoint],
    });
    const matches: MatchEntity[] = await matchQueryHandler.execute(matchQuery);
    expect(matches.length).toBeGreaterThanOrEqual(0);
  });
});
